@@ -1,8 +1,8 @@
-from collections.abc import Iterable
+import copy
 import os
+import pickle
 import typing
 
-import numpy as np
 import pandas as pd
 
 import audeer
@@ -19,39 +19,15 @@ from audformat.core.errors import (
 )
 from audformat.core.index import (
     filewise_index,
-    segmented_index,
     index_type,
 )
+from audformat.core.media import Media
+from audformat.core.split import Split
 from audformat.core.typing import (
     Values,
 )
 
 
-def index_to_dict(index: pd.Index) -> dict:
-    r"""Convert :class:`pandas.Index` to a dictionary.
-
-    Returns a dictionary with keys files, starts, and ends.
-
-    """
-    d = dict(
-        [
-            (define.IndexField.FILE + 's', None),
-            (define.IndexField.START + 's', None),
-            (define.IndexField.END + 's', None),
-        ]
-    )
-
-    d[define.IndexField.FILE + 's'] = index.get_level_values(
-        define.IndexField.FILE).values
-    if index_type(index) == define.IndexType.SEGMENTED:
-        d[define.IndexField.START + 's'] = index.get_level_values(
-            define.IndexField.START).values.astype(np.timedelta64)
-        d[define.IndexField.END + 's'] = index.get_level_values(
-            define.IndexField.END).values.astype(np.timedelta64)
-
-    return d
-
-
 class Table(HeaderBase):
     r"""Table with annotation data.
 
@@ -104,6 +80,12 @@ class Table(HeaderBase):
         file
         f1        0
         f2        1
+        >>> table.get(as_segmented=True)
+                        values
+        file start  end
+        f1   0 days NaT      0
+        f2   0 days NaT      1
+        f3   0 days NaT      2
         >>> index_ex = filewise_index('f4')
         >>> table_ex = table.extend_index(
         ...     index_ex,
@@ -145,8 +127,6 @@ class Table(HeaderBase):
         f4        3     NaN
 
     """
-    TYPE = 'type'
-
     def __init__(
             self,
             index: pd.Index = None,
@@ -178,6 +158,16 @@ class Table(HeaderBase):
         self._db = None
         self._id = None
 
+    @property
+    def db(self):
+        r"""Database object.
+
+        Returns:
+            database object or ``None`` if not assigned yet
+
+        """
+        return self._db
+
     @property
     def df(self) -> pd.DataFrame:
         r"""Table data.
@@ -186,6 +176,11 @@ class Table(HeaderBase):
             data
 
         """
+        if self._df is None:
+            # if database was loaded with 'load_data=False'
+            # we have to load the table data now
+            path = os.path.join(self.db.root, f'{self.db._name}.{self._id}')
+            self.load(path)
         return self._df
 
     @property
@@ -213,7 +208,8 @@ class Table(HeaderBase):
             files
 
         """
-        if self.df.index.empty:
+        # We use len() here as self.df.index.empty takes a very long time
+        if len(self.df.index) == 0:
             return filewise_index()
         else:
             index = self.df.index.get_level_values(define.IndexField.FILE)
@@ -228,7 +224,7 @@ class Table(HeaderBase):
             index
 
         """
-        return self._df.index
+        return self.df.index
 
     @property
     def is_filewise(self) -> bool:
@@ -250,6 +246,28 @@ class Table(HeaderBase):
         """
         return self.type == define.IndexType.SEGMENTED
 
+    @property
+    def media(self) -> typing.Optional[Media]:
+        r"""Media object.
+
+        Returns:
+            media object or ``None`` if not available
+
+        """
+        if self.media_id is not None and self.db is not None:
+            return self.db.media[self.media_id]
+
+    @property
+    def split(self) -> typing.Optional[Split]:
+        r"""Split object.
+
+        Returns:
+            split object or ``None`` if not available
+
+        """
+        if self.split_id is not None and self.db is not None:
+            return self.db.splits[self.split_id]
+
     @property
     def starts(self) -> pd.Index:
         r"""Segment start times.
@@ -275,11 +293,11 @@ class Table(HeaderBase):
 
         """
         table = Table(
-            self._df.index,
+            self.df.index,
             media_id=self.media_id,
             split_id=self.split_id,
         )
-        table._db = self._db
+        table._db = self.db
         for column_id, column in self.columns.items():
             table.columns[column_id] = Column(
                 scheme_id=column.scheme_id,
@@ -287,7 +305,7 @@ class Table(HeaderBase):
                 description=column.description,
                 meta=column.meta.copy()
             )
-        table._df = self._df.copy()
+        table._df = self.df.copy()
         return table
 
     def drop_columns(
@@ -349,11 +367,11 @@ class Table(HeaderBase):
             files = [files]
         if callable(files):
             sel = self.files.to_series().apply(files)
-            self._df = self._df[~sel.values]
+            self._df = self.df[~sel.values]
         else:
             index = self.files.intersection(files)
             index.name = define.IndexField.FILE
-            self._df.drop(index, inplace=True)
+            self.df.drop(index, inplace=True)
 
         return self
 
@@ -389,8 +407,8 @@ class Table(HeaderBase):
                 f'{self.type} '
                 'index'
             )
-        new_index = self._df.index.difference(index)
-        self._df = self._df.reindex(new_index)
+        new_index = self.df.index.difference(index)
+        self._df = self.df.reindex(new_index)
 
         return self
 
@@ -434,14 +452,14 @@ class Table(HeaderBase):
                 f'{input_type} '
                 f'index.'
             )
-        new_index = self._df.index.union(index)
-        self._df = self._df.reindex(new_index)
+        new_index = self.df.index.union(index)
+        self._df = self.df.reindex(new_index)
         if fill_values is not None:
             if isinstance(fill_values, dict):
                 for key, value in fill_values.items():
-                    self._df[key].fillna(value, inplace=True)
+                    self.df[key].fillna(value, inplace=True)
             else:
-                self._df.fillna(fill_values, inplace=True)
+                self.df.fillna(fill_values, inplace=True)
 
         return self
 
@@ -453,6 +471,11 @@ class Table(HeaderBase):
                 str, typing.Union[str, typing.Sequence[str]]
             ] = None,
             copy: bool = True,
+            as_segmented: bool = False,
+            allow_nat: bool = True,
+            root: str = None,
+            num_workers: typing.Optional[int] = 1,
+            verbose: bool = False,
     ) -> pd.DataFrame:
         r"""Get labels.
 
@@ -475,11 +498,30 @@ class Table(HeaderBase):
                 values to age and gender, respectively.
                 To also keep the original column with speaker IDS, you can do
                 ``map={'speaker': ['speaker', 'age', 'gender']}``
+            as_segmented: if set to ``True``
+                and table has a filewise index,
+                the index of the returned table
+                will be converted to a segmented index.
+                ``start`` will be set to ``0`` and
+                ``end`` to ``NaT`` or to the file duration
+                if ``allow_nat`` is set to ``False``
+            allow_nat: if set to ``False``,
+                ``end=NaT`` is replaced with file duration
+            root: root directory under which the files are stored.
+                Provide if file names are relative and
+                database was not saved or loaded from disk.
+                If ``None`` :attr:`audformat.Database.root` is used.
+                Only relevant if ``allow_nat`` is set to ``False``
+            num_workers: number of parallel jobs.
+                If ``None`` will be set to the number of processors
+                on the machine multiplied by 5
+            verbose: show progress bar
 
         Returns:
             labels
 
         Raises:
+            FileNotFoundError: if file is not found
             RuntimeError: if table is not assign to a database
             ValueError: if trying to map without a scheme
             ValueError: if trying to map from a scheme that has no labels
@@ -489,28 +531,29 @@ class Table(HeaderBase):
         result_is_copy = False
 
         if index is None:
-            result = self._df
+            result = self.df
         else:
             if index_type(self.index) == index_type(index):
-                result = self._df.loc[index]
+                result = self.df.loc[index]
             else:
                 files = index.get_level_values(define.IndexField.FILE)
                 if self.is_filewise:  # index is segmented
                     result = pd.DataFrame(
-                        self._df.loc[files].values,
+                        self.df.loc[files].values,
                         index,
                         columns=self.columns
                     )
                     result_is_copy = True  # to avoid another copy
                 else:  # index is filewise
                     files = list(dict.fromkeys(files))  # remove duplicates
-                    result = self._df.loc[files]
+                    result = self.df.loc[files]
 
         if map is not None:
 
-            if self._db is None:
+            if self.db is None:
                 raise RuntimeError(
-                    'Table is not assigned to a database.'
+                    'Cannot map schemes, '
+                    'table is not assigned to a database.'
                 )
 
             if not result_is_copy:
@@ -532,6 +575,26 @@ class Table(HeaderBase):
                 if column not in mapped_columns:
                     result.drop(columns=column, inplace=True)
 
+        # if necessary, convert to segmented index and replace NaT
+        is_segmented = index_type(result.index) == define.IndexType.SEGMENTED
+        if (
+                (not is_segmented and as_segmented)
+                or (is_segmented and not allow_nat)
+        ):
+            files_duration = None
+            if self.db is not None:
+                files_duration = self.db._files_duration
+                root = root or self.db.root
+            new_index = utils.to_segmented_index(
+                result.index,
+                allow_nat=allow_nat,
+                files_duration=files_duration,
+                root=root,
+                num_workers=num_workers,
+                verbose=verbose,
+            )
+            result = result.set_axis(new_index)
+
         return result.copy() if (copy and not result_is_copy) else result
 
     def load(
@@ -540,16 +603,53 @@ class Table(HeaderBase):
     ):
         r"""Load table data from disk.
 
+        Tables can be stored as PKL and/or CSV files to disk.
+        If both files are present
+        it will load the PKL file
+        as long as its modification date is newer,
+        otherwise it will raise an error
+        and ask to delete one of the files.
+
         Args:
             path: file path without extension
 
+        Raises:
+            RuntimeError: if table file(s) are missing
+            RuntimeError: if CSV file is newer than PKL file
+
         """
         path = audeer.safe_path(path)
-        pickled = os.path.exists(path + f'.{define.TableStorageFormat.PICKLE}')
+        pkl_file = f'{path}.{define.TableStorageFormat.PICKLE}'
+        csv_file = f'{path}.{define.TableStorageFormat.CSV}'
+
+        if not os.path.exists(pkl_file) and not os.path.exists(csv_file):
+            raise RuntimeError(
+                f"No file found for table with path '{path}.{{pkl|csv}}'"
+            )
+
+        # Load from PKL if file exists and is newer then CSV file.
+        # If both are written by Database.save() this is the case
+        # as it stores first the PKL file
+        pickled = False
+        if os.path.exists(pkl_file):
+            if (
+                    os.path.exists(csv_file)
+                    and os.path.getmtime(csv_file) > os.path.getmtime(pkl_file)
+            ):
+                raise RuntimeError(
+                    f"The table CSV file '{csv_file}' is newer "
+                    f"than the table PKL file '{pkl_file}'. "
+                    "If you want to load from the CSV file, "
+                    "please delete the PKL file. "
+                    "If you want to load from the PKL file, "
+                    "please delete the CSV file."
+                )
+            pickled = True
+
         if pickled:
-            self._load_pickled(path + f'.{define.TableStorageFormat.PICKLE}')
+            self._load_pickled(pkl_file)
         else:
-            self._load_csv(path + f'.{define.TableStorageFormat.CSV}')
+            self._load_csv(csv_file)
 
     def pick_columns(
             self,
@@ -609,8 +709,8 @@ class Table(HeaderBase):
                 f'{self.type} '
                 f'index'
             )
-        new_index = self._df.index.intersection(index)
-        self._df = self._df.reindex(new_index)
+        new_index = self.df.index.intersection(index)
+        self._df = self.df.reindex(new_index)
 
         return self
 
@@ -643,7 +743,7 @@ class Table(HeaderBase):
             files = [files]
         if callable(files):
             sel = self.files.to_series().apply(files)
-            self._df = self._df[sel.values]
+            self._df = self.df[sel.values]
         else:
             index = self.files.intersection(files)
             index.name = define.IndexField.FILE
@@ -655,29 +755,40 @@ class Table(HeaderBase):
             self,
             path: str,
             *,
-            storage_format: define.TableStorageFormat = (
-                define.TableStorageFormat.CSV
-            ),
+            storage_format: str = define.TableStorageFormat.CSV,
+            update_other_formats: bool = True,
     ):
         r"""Save table data to disk.
 
+        Existing files will be overwritten.
+
         Args:
             path: file path without extension
-            storage_format: storage format of tables.
+            storage_format: storage format of table.
                 See :class:`audformat.define.TableStorageFormat`
                 for available formats
+            update_other_formats: if ``True`` it will not only save
+                to the given ``storage_format``,
+                but update all files stored in other storage formats as well
 
         """
         path = audeer.safe_path(path)
-        define.TableStorageFormat.assert_has_value(storage_format)
+        define.TableStorageFormat.assert_has_attribute_value(storage_format)
+
+        pickle_file = path + f'.{define.TableStorageFormat.PICKLE}'
+        csv_file = path + f'.{define.TableStorageFormat.CSV}'
+
+        # Make sure the CSV file is always written first
+        # as it is expected to be older by load()
         if storage_format == define.TableStorageFormat.PICKLE:
-            self._df.to_pickle(
-                path + f'.{define.TableStorageFormat.PICKLE}',
-                compression='xz',
-            )
-        else:
-            with open(path + f'.{define.TableStorageFormat.CSV}', 'w') as fp:
-                self.df.to_csv(fp, encoding='utf-8')
+            if update_other_formats and os.path.exists(csv_file):
+                self._save_csv(csv_file)
+            self._save_pickled(pickle_file)
+
+        if storage_format == define.TableStorageFormat.CSV:
+            self._save_csv(csv_file)
+            if update_other_formats and os.path.exists(pickle_file):
+                self._save_pickled(pickle_file)
 
     def set(
             self,
@@ -710,101 +821,221 @@ class Table(HeaderBase):
         for idx, data in values.items():
             self.columns[idx].set(data, index=index)
 
+    def update(
+            self,
+            others: typing.Union['Table', typing.Sequence['Table']],
+            *,
+            overwrite: bool = False,
+    ) -> 'Table':
+        r"""Update table with other table(s).
+
+        Table which calls ``update()`` must be assigned to a database.
+        For all tables media and split must match.
+
+        Columns that are not yet part of the table will be added and
+        referenced schemes or raters are copied.
+        For overlapping columns, schemes and raters must match.
+
+        Columns with the same identifier are combined to a single column.
+        This requires that both columns have the same dtype
+        and if ``overwrite`` is set to ``False``,
+        values in places where the indices overlap have to match
+        or one column contains ``NaN``.
+        If ``overwrite`` is set to ``True``,
+        the value of the last table in the list is kept.
+
+        The index type of the table must not change.
+
+        Args:
+            others: table object(s)
+            overwrite: overwrite values where indices overlap
+
+        Returns:
+            the updated table
+
+        Raises:
+            RuntimeError: if table is not assign to a database
+            ValueError: if split or media does not match
+            ValueError: if overlapping columns reference different schemes
+                or raters
+            ValueError: if a missing scheme or rater cannot be copied
+                because a different object with the same ID exists
+            ValueError: if values in same position overlap
+            ValueError: if operation would change the index type of the table
+
+        """
+        if self.db is None:
+            raise RuntimeError(
+                'Table is not assigned to a database.'
+            )
+
+        if isinstance(others, Table):
+            others = [others]
+
+        for other in others:
+            if self.type != other.type:
+                raise ValueError(
+                    'Cannot update a '
+                    f'{self.type} '
+                    'table with a '
+                    f'{other.type} '
+                    'table.'
+                )
+
+        def raise_error(
+                msg,
+                left: typing.Optional[HeaderDict],
+                right: typing.Optional[HeaderDict],
+        ):
+            raise ValueError(
+                f"{msg}:\n"
+                f"{left}\n"
+                "!=\n"
+                f"{right}"
+            )
+
+        def assert_equal(
+                msg: str,
+                left: typing.Optional[HeaderDict],
+                right: typing.Optional[HeaderDict],
+        ):
+            equal = True
+            if left and right:
+                equal = left == right
+            elif left or right:
+                equal = False
+            if not equal:
+                raise_error(msg, left, right)
+
+        missing_schemes = {}
+        missing_raters = {}
+
+        for other in others:
+
+            assert_equal(
+                "Media of table "
+                f"'{other._id}' "
+                "does not match",
+                self.media,
+                other.media,
+            )
+
+            assert_equal(
+                "Split of table "
+                f"'{other._id}' "
+                "does not match",
+                self.split,
+                other.split,
+            )
+
+            # assert schemes match for overlapping columns and
+            # look for missing schemes in new columns,
+            # raise an error if a different scheme with same ID exists
+            for column_id, column in other.columns.items():
+                if column_id in self.columns:
+                    assert_equal(
+                        "Scheme of common column "
+                        f"'{other._id}.{column_id}' "
+                        "does not match",
+                        self.columns[column_id].scheme,
+                        column.scheme,
+                    )
+                else:
+                    if column.scheme is not None:
+                        if column.scheme_id in self.db.schemes:
+                            assert_equal(
+                                "Cannot copy scheme of column "
+                                f"'{other._id}.{column_id}' "
+                                "as a different scheme with ID "
+                                f"'{column.scheme_id}' "
+                                "exists",
+                                self.db.schemes[column.scheme_id],
+                                column.scheme,
+                            )
+                        else:
+                            missing_schemes[column.scheme_id] = column.scheme
+
+            # assert raters match for overlapping columns and
+            # look for missing raters in new columns,
+            # raise an error if a different rater with same ID exists
+            for column_id, column in other.columns.items():
+                if column_id in self.columns:
+                    assert_equal(
+                        f"self['{self._id}']['{column_id}'].rater "
+                        "does not match "
+                        f"other['{other._id}']['{column_id}'].rater",
+                        self.columns[column_id].rater,
+                        column.rater,
+                    )
+                else:
+                    if column.rater is not None:
+                        if column.rater_id in self.db.raters:
+                            assert_equal(
+                                f"db1.raters['{column.scheme_id}'] "
+                                "does not match "
+                                f"db2.raters['{column.scheme_id}']",
+                                self.db.raters[column.rater_id],
+                                column.rater,
+                            )
+                        else:
+                            missing_raters[column.rater_id] = column.rater
+
+        # concatenate table data
+        df = utils.concat(
+            [self.df] + [other.df for other in others],
+            overwrite=overwrite,
+        )
+
+        # insert missing schemes and raters
+        for scheme_id, scheme in missing_schemes.items():
+            self.db.schemes[scheme_id] = copy.copy(scheme)
+        for rater_id, rater in missing_raters.items():
+            self.db.raters[rater_id] = copy.copy(rater)
+
+        # insert new columns
+        for other in others:
+            for column_id, column in other.columns.items():
+                if column_id not in self.columns:
+                    self.columns[column_id] = copy.copy(column)
+
+        # update table data
+        self._df = df
+
+        return self
+
     def __add__(self, other: 'Table') -> 'Table':
         r"""Create new table by combining two tables.
 
-        If the tables are of the same type, the created tables contain index
-        and columns of both tables.
-        If the tables are not of the same type and one of the tables is a
-        file-wise table, the created table has the index of segmented table
-        and columns of both tables.
+        The new table contains index and columns of both tables.
         Missing values will be set to ``NaN``.
-        References to schemes and raters are always preserved.
-        Media and split information only when they match.
+        If at least one table is segmented, the output has a segmented index.
 
-        .. warning::
-            Columns with the same identifier are combined to a single column.
-            This requires that either the indices of the tables do not
-            overlap or at least one table contains a ``NaN`` in places where
-            the indices overlap.
+        Columns with the same identifier are combined to a single column.
+        This requires that:
+
+        1. both columns have the same dtype
+        2. in places where the indices overlap the values of both columns
+           match or one column contains ``NaN``
+
+        Media and split information,
+        as well as,
+        references to schemes and raters are discarded.
+        If you intend to keep them,
+        use :meth:`audformat.Table.update`.
 
         Args:
             other: the other table
 
         Raises:
-            ValueError: if overlapping values are detected that are not ``NaN``
+            ValueError: if columns with the same name have different dtypes
+            ValueError: if values in the same position do not match
 
         """
-        if self == other:
-            return self
+        df = utils.concat([self.df, other.df])
 
-        df_self = self._df
-        df_other = other._df
-        df_other_copy = False
-
-        if other.type != self.type:
-
-            def add_files_to_dict(d, additional_files, table_type):
-                if not additional_files.empty:
-                    d['files'] = np.r_[d['files'], additional_files.values]
-                    if table_type == define.IndexType.SEGMENTED:
-                        d_append = index_to_dict(
-                            utils.to_segmented_index(additional_files)
-                        )
-                        d['starts'] = np.r_[d['starts'], d_append['starts']]
-                        d['ends'] = np.r_[d['ends'], d_append['ends']]
-
-            if other.type == define.IndexType.FILEWISE:
-                missing_files = other.files.unique().difference(
-                    self.files.unique()
-                )
-                d = index_to_dict(self._df.index)
-                add_files_to_dict(d, missing_files, self.type)
-                df_other = other._df.reindex(d['files'])
-                df_other.set_index(segmented_index(**d), inplace=True)
-            elif self.type == define.IndexType.FILEWISE:
-                missing_files = self.files.unique().difference(
-                    other.files.unique()
-                )
-                d = index_to_dict(other._df.index)
-                add_files_to_dict(d, missing_files, other.type)
-                df_self = self._df.reindex(d['files'])
-                df_self.set_index(segmented_index(**d), inplace=True)
-
-        # figure out column names, schemes and raters
-        df_columns = []
-        scheme_ids = {}
-        rater_ids = {}
-        for column_id, column in self.columns.items():
-            df_columns.append(column_id)
-            scheme_ids[column_id] = column.scheme_id
-            rater_ids[column_id] = column.rater_id
-        for column_id, column in other.columns.items():
-            if column_id in df_self.columns:
-                if not df_self.index.intersection(df_other.index).empty:
-                    if not df_other_copy:
-                        df_other = df_other.copy()
-                        df_other_copy = True
-                    df_other.update(df_self, errors='raise')
-            else:
-                df_columns.append(column_id)
-                scheme_ids[column_id] = column.scheme_id
-                rater_ids[column_id] = column.rater_id
-
-        # concatenate frames
-        df = utils.concat([df_self, df_other])
-        df.columns = df_columns
-
-        # create table
-        media_id = self.media_id if self.media_id == other.media_id else None
-        split_id = self.split_id if self.split_id == other.split_id else None
-        table = Table(df.index, media_id=media_id, split_id=split_id)
-        table._db = self._db
-        for column_id in df_columns:
-            table[column_id] = Column(
-                scheme_id=scheme_ids[column_id],
-                rater_id=rater_ids[column_id],
-            )
+        table = Table(df.index)
+        for column_id in df:
+            table[column_id] = Column()
         table._df = df
 
         return table
@@ -818,8 +1049,16 @@ class Table(HeaderBase):
         """
         return self.columns[column_id]
 
+    def __eq__(
+            self,
+            other: 'Table',
+    ) -> bool:
+        if self.dump() != other.dump():
+            return False
+        return self.df.equals(other.df)
+
     def __len__(self) -> int:
-        return len(self._df)
+        return len(self.df)
 
     def __setitem__(self, column_id: str, column: Column) -> Column:
         r"""Add new column to table.
@@ -841,7 +1080,7 @@ class Table(HeaderBase):
         usecols = []
         dtypes = {}
         converters = {}
-        schemes = self._db.schemes
+        schemes = self.db.schemes
 
         # index columns
 
@@ -864,12 +1103,11 @@ class Table(HeaderBase):
             usecols.append(column_id)
             if column.scheme_id is not None:
                 dtype = schemes[column.scheme_id].to_pandas_dtype()
-                # if column contains timestamps
-                # we have to convert them later
-                if dtype == 'timedelta64[ns]':
-                    converters[column_id] = lambda x: pd.to_timedelta(x)
-                elif dtype == 'datetime64[ns]':
+                # use converter if column contains dates or timestamps
+                if dtype == 'datetime64[ns]':
                     converters[column_id] = lambda x: pd.to_datetime(x)
+                elif dtype == 'timedelta64[ns]':
+                    converters[column_id] = lambda x: pd.to_timedelta(x)
                 else:
                     dtypes[column_id] = dtype
             else:
@@ -883,13 +1121,22 @@ class Table(HeaderBase):
             dtype=dtypes,
             index_col=index_col,
             converters=converters,
+            float_precision='round_trip',
         )
 
         self._df = df
 
     def _load_pickled(self, path: str):
 
-        df = pd.read_pickle(path, compression='xz')
+        # Older versions of audformat used xz compression
+        # which produced smaller files,
+        # but was slower.
+        # The try-except statement allows backward compatibility
+        try:
+            df = pd.read_pickle(path)
+        except pickle.UnpicklingError:
+            df = pd.read_pickle(path, compression='xz')
+
         for column_id in df:
             # Categories of type Int64 are somehow converted to int64.
             # We have to change back to Int64 to make column nullable.
@@ -906,26 +1153,37 @@ class Table(HeaderBase):
 
         self._df = df
 
+    def _save_csv(self, path: str):
+        # Load table before opening CSV file
+        # to avoid creating a CSV file
+        # that is newer than the PKL file
+        df = self.df
+        with open(path, 'w') as fp:
+            df.to_csv(fp, encoding='utf-8')
+
+    def _save_pickled(self, path: str):
+        self.df.to_pickle(path)
+
     def _set_column(self, column_id: str, column: Column) -> Column:
         if column.scheme_id is not None and \
-                column.scheme_id not in self._db.schemes:
-            raise BadIdError('column', column.scheme_id, self._db.schemes)
+                column.scheme_id not in self.db.schemes:
+            raise BadIdError('column', column.scheme_id, self.db.schemes)
         if column.rater_id is not None and \
-                column.rater_id not in self._db.raters:
-            raise BadIdError('rater', column.rater_id, self._db.raters)
+                column.rater_id not in self.db.raters:
+            raise BadIdError('rater', column.rater_id, self.db.raters)
 
         if column.scheme_id is not None:
-            dtype = self._db.schemes[column.scheme_id].to_pandas_dtype()
+            dtype = self.db.schemes[column.scheme_id].to_pandas_dtype()
         else:
             dtype = object
-        self._df[column_id] = pd.Series(dtype=dtype)
+        self.df[column_id] = pd.Series(dtype=dtype)
 
         #  if table is empty we need to fix index names
-        if self._df.empty:
+        if self.df.empty:
             if self.is_filewise:
-                self._df.index.name = define.IndexField.FILE
+                self.df.index.name = define.IndexField.FILE
             elif self.is_segmented:
-                self._df.index.rename(
+                self.df.index.rename(
                     [
                         define.IndexField.FILE,
                         define.IndexField.START,
